require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.provider.ContactsContract"
import "android.net.Uri"
import "android.Manifest"
import "android.content.pm.PackageManager"
import "android.content.Intent"
import "java.util.*"

local activity = activity
local contactList = {}

function onCreate(savedInstanceState)
    activity.setTitle("Speed Dial")
    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    -- Request contacts permissions
    if not checkContactPermissions() then
        requestContactPermissions()
    else
        loadContacts(layout)
    end

    activity.setContentView(layout)
end

function checkContactPermissions()
    return activity.checkSelfPermission(Manifest.permission.READ_CONTACTS) == PackageManager.PERMISSION_GRANTED
end

function requestContactPermissions()
    activity.requestPermissions({ Manifest.permission.READ_CONTACTS }, 1)
end

function loadContacts(layout)
    -- Create the contact selection dropdown
    local contactsSpinner = Spinner(activity)
    layout.addView(contactsSpinner)

    -- Create the call button
    local callButton = Button(activity)
    callButton.setText("Call")
    callButton.setOnClickListener{
        onClick = function(v)
            local selectedPosition = contactsSpinner.getSelectedItemPosition()
            local phoneNumber = contactList[selectedPosition + 1].phoneNumber
            callContact(phoneNumber)
        end
    }
    layout.addView(callButton)

    -- Load contacts
    populateContacts(contactsSpinner)
end

function populateContacts(contactsSpinner)
    local cursor = activity.getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, nil, nil, nil, nil)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, {})
    contactsSpinner.setAdapter(adapter)

    while cursor.moveToNext() do
        local id = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
        local name = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts.DISPLAY_NAME))
        local phoneNumber = getPhoneNumber(id)

        if phoneNumber then
            adapter.add(name)
            table.insert(contactList, {name = name, phoneNumber = phoneNumber})
        end
    end
    cursor.close()
    adapter.notifyDataSetChanged()
end

function getPhoneNumber(contactId)
    local cursor = activity.getContentResolver().query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, nil, ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?", {contactId}, nil)
    if cursor.moveToFirst() then
        local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        cursor.close()
        return phoneNumber
    end
    cursor.close()
    return nil
end

function callContact(phoneNumber)
    local intent = Intent(Intent.ACTION_CALL)
    intent.setData(Uri.parse("tel:" .. phoneNumber))
    activity.startActivity(intent)
end